@@ -270,7 +270,9 @@ class Project:
                 return self.python
 
         if self.root.joinpath("__pypackages__").exists() or not config["python.use_venv"] or self.is_global:
-            for py_version in self.iter_interpreters(filter_func=match_version):
+            for py_version in self.iter_interpreters(
+                filter_func=match_version, respect_version_file=config["python.use_python_version"]
+            ):
                 note("[success]__pypackages__[/] is detected, using the PEP 582 mode")
                 self.python = py_version
                 return py_version
@@ -426,13 +428,16 @@ class Project:
         extra_deps: dict[str, list[Requirement]] = {}
         while referred_groups:
             updated = False
-            for group, referred in list(referred_groups.items()):
+            ref_iter = list(referred_groups.items())
+            for group, referred in ref_iter:
                 for ref in list(referred):
                     if ref not in requested_groups:
                         deps, r = _get_dependencies(ref)
                         group_deps[ref] = deps
                         if r:
                             referred_groups[ref] = r
+                            # append to the ref_iter to process later
+                            ref_iter.append((ref, r))
                         requested_groups.append(ref)
                     if ref in referred_groups:  # not resolved yet
                         continue
@@ -445,7 +450,7 @@ class Project:
                 if not referred:
                     referred_groups.pop(group)
             if not updated:
-                raise ProjectError("Cyclic dependency group include detected")
+                raise ProjectError(f"Cyclic dependency group include detected: {set(referred_groups)}")
         if include_referred:
             for group, deps in extra_deps.items():
                 group_deps[group].extend(deps)
@@ -798,16 +803,19 @@ class Project:
 
         from pdm.cli.commands.python import InstallCommand
 
+        def read_version_from_version_file(python_version_file: Path) -> str | None:
+            content = python_version_file.read_text().strip()
+            content_lines = [cl for cl in content.splitlines() if not cl.lstrip().startswith("#")]
+
+            return content_lines[0] if len(content_lines) == 1 else None
+
+        version_file = self.root.joinpath(".python-version")
         found = False
-        if (
-            respect_version_file
-            and not python_spec
-            and (os.getenv("PDM_PYTHON_VERSION") or self.root.joinpath(".python-version").exists())
-        ):
-            requested = os.getenv("PDM_PYTHON_VERSION") or self.root.joinpath(".python-version").read_text().strip()
-            if requested not in self.python_requires:
+        if respect_version_file and not python_spec and (os.getenv("PDM_PYTHON_VERSION") or version_file.exists()):
+            requested = os.getenv("PDM_PYTHON_VERSION") or read_version_from_version_file(version_file)
+            if requested is not None and requested not in self.python_requires:
                 self.core.ui.warn(".python-version is found but the version is not in requires-python, ignored.")
-            else:
+            elif requested is not None:
                 python_spec = requested
         for interpreter in self.find_interpreters(python_spec, search_venv):
             if filter_func is None or filter_func(interpreter):
